"""$Id: """
#######################################################################
#                                                                     #
#                  copyright (c) 2003 Ingeniweb SARL                  #
#                                                                     #
#######################################################################

"""PloneSeleniumTool main class"""

# Python imports
import os
from urlparse import urlparse

# Zope imports
from AccessControl import ClassSecurityInfo
from AccessControl import Unauthorized
from Globals import DTMLFile
from Globals import InitializeClass
from OFS.Folder import Folder
from OFS.PropertyManager import PropertyManager
from zExceptions import NotFound
from Products.CMFCore.utils import UniqueObject
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

# CMF imports
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.Expression import createExprContext
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.Expression import createExprContext
from Products.PloneSelenium.Interpreter import PloneSeleniumInterpreter


from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManager import setSecurityPolicy
from Products.CMFCore.tests.base.security import PermissiveSecurityPolicy, OmnipotentUser



_www = os.path.join(os.path.dirname(__file__), 'www')
_defaultstartdate={'years': '2005', 'months': 'June', 'days': '30',
                                'hours': '10', 'minutes': '00'}
_defaultenddate={'years': '2005', 'months': 'June', 'days': '30',
                                'hours': '20', 'minutes': '00'}

def extractSuiteScriptName(action):
    import re
    match = re.search('get_.*_ftests', action)
    return match.group()              
                               
class FTest(PloneSeleniumInterpreter):
    """ TODO: add documentation
    """
    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, base, language='en'):
        """ Initializes a functional test

        'base' -- TODO what is it?
        """
        self.base = base
        self.ftest=[]
        self.language = language

    def __add__(self, other):
        return self.ftest + other

    def __radd__(self, other):
        return  other + self.ftest

    def __repr__(self):
        return repr(self.ftest)

    def dispatchCommand(self, command, target="", value=""):
        if command in ('open','verifyLocation'):
            target = self.base + target
        self.ftest.append((command, target, value,))
        

class Suite(PloneSeleniumInterpreter):
    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self):
        self.suite = []
        self.language = 'en'

    def setBase(self, base):
        self.base = base

    def getBase(self):
        return self.base

    def setTargetLanguage(self, language):
        self.language = language

    def add(self, title, steps):
        """Add Sequence

        'title' is the name of the test sequence and all the other
        arguments are searched for strings: they signify the start of
        a new test step, all the step arguments till the next string
        are added to the step.
        """
        sequence=[]
        test={'title':title, 'steps':[]}
        test_steps=[]
        test_title=""
        for element in steps:
            if type(element)==type(""):
                if test_steps:
                    # Finish last step
                    sequence.append({'title':test_title, 'steps':test_steps})
                test_title=element # Fresh start
                test_steps=[]
            else:
                test_steps.append(element)
        if test_steps:
            # Finish last step
            sequence.append({'title':test_title, 'steps':test_steps})
        self.suite.append({'title':title, 'tests':sequence})


    def addTests(self, title, *steps):
        """Add Sequence
        """
        sequence=[]
        test={'title':title, 'steps':[]}
        test_steps=[]
        test_title=""
        for element in steps:
            if type(element)==type(""):
                if test_steps:
                    # Finish last step
                    if test_title:
                        sequence.append({'title':test_title, 'steps':test_steps})
                    else:
                        # Take the title of the whole test as a fallback
                        sequence.append({'title':title, 'steps':test_steps})
                test_title=element # Fresh start
                test_steps=[]
            elif hasattr(element, 'ftest'):
                for e in element.ftest:
                    test_steps.append(e)
            else:
                test_steps.append(element)

        if test_steps:
            # Finish last step
            sequence.append({'title':test_title, 'steps':test_steps})
        self.suite.append({'title':title, 'tests':sequence})

    def getTest(self):
        return FTest(self.base, self.language)

    #
    #   Test methods
    #

    def dispatchCommand(self, command, target="", value=""):
        if command in ('open','verifyLocation'):
            target = self.base + target
        return (command, target, value,)

    def test_logout(self):
        t = self.getTest()
        t.open("/logout")
        return t

    TestLogout = test_logout # TestLogout is deprecated

    def test_login_portlet(self, login=''):
        t = self.getTest()
        t.open("/logout")
        t.type("__ac_name", login)
        t.type("__ac_password", "admin")
        t.clickAndWait("submit")
        t.verifyTextPresent(login)
        return t

    TestLoginPortlet = test_login_portlet # TestLoginPortlet is deprecated

    def test_switch_to_language(self, language='en', verify_text=''):
        t = self.getTest()
        # "Test Switch to Language"
        t.open('/')
        t.open("/index_html?set_language=%s" % language)
        if verify_text:
            t.verifyTextPresent(verify_text)
        return t

    TestSwitchToLanguage = test_switch_to_language # TestSwitchToLanguage deprecated

    def test_access_url_verify_text(self, url, text):
        t = self.getTest()
        # "Access %s and Verify Text %s" % (url, text)
        t.open(url)
        t.verifyTextPresent(text)
        return t

    TestAccessUrlVerifyText  = test_access_url_verify_text # TestLogout is deprecated

    def test_add_folder(self, folder='/', id='folder', title='folder title'):
        """Add a minimal Folder to the folder
        """       
        t = self.getTest()
        if not folder.endswith('/'):
            folder += '/'
        t.open(folder+"createObject?type_name=Folder")
        t.type("id", id)
        t.type("title", title)
        t.clickAndWait( "form_submit")
        return t

    def test_add_page(self, folder='/', id=None, title='page title', text='page text'):
        """Add a minimal Page to the folder
        """       
        t = self.getTest()
        t.open(folder)
        t.open(folder+"/createObject?type_name=Document")  # "//a[@title='A page in the portal, which can contain rich text.']")
        if id:
            t.type("id", id)
        t.type("title", title)
        t.type("text",text)
        t.clickAndWait( "form_submit")
        t.verifyTextPresent("Your changes have been saved.")
        return t

    def test_delete_object(self, folder='/', id='page'):
        """Delete an object from a folder
        """
        t = self.getTest()
        t.open("%s/folder_contents" % folder)
        t.click("//input[@id='cb_%s']" % id)
        t.clickAndWait("//input[@value='Delete']")
        t.open('%s/%s' % (folder, id))
        t.verifyTextPresent('Our apologies')
        return t

    def test_add_newsitem(self, folder='/', id='newsitem', title='news item title', text='news item text'):
        """Add a minimal News Item to the folder
        """       
        t = self.getTest()
        t.open(folder)
        if not folder.endswith('/'):
            folder += '/'
        
        t.open(folder+"createObject?type_name=News Item")
        t.type("id", id)
        t.type("title", title)
        t.type("text",text)
        t.clickAndWait( "form_submit")
        return t

    def test_add_link(self, folder='/', id='link', title='link title', url='http://localhost/'):
        """Add a minimal Link to the folder
        """       
        t = self.getTest()
        t.open(folder)
        if not folder.endswith('/'):
            folder += '/'

        t.open(folder+"createObject?type_name=Link")
        t.type("id", id)
        t.type("title", title)
        t.type("remoteUrl",url)
        t.clickAndWait( "form_submit")
        return t
        
    def test_add_event(self, folder='/', id='event', title='event title', 
            startDate=_defaultstartdate, endDate=_defaultenddate,
            eventtype='Work'):
        """Add a minimal Event to the folder
        """       
        t = self.getTest()
        t.open(folder)
        if not folder.endswith('/'):
            folder += '/'

        t.open(folder+"createObject?type_name=Event")
        t.type("id", id)
        t.type("title", title)
        t.select("edit_form_startDate_3_year",startDate['years'])
        t.select("edit_form_startDate_3_month",startDate['months'])
        t.select("edit_form_startDate_3_day",startDate['days'])
        t.select("edit_form_startDate_3_hour",startDate['hours'])
        t.select("edit_form_startDate_3_minute",startDate['minutes'])
        t.select("edit_form_endDate_4_year",endDate['years'])
        t.select("edit_form_endDate_4_month",endDate['months'])
        t.select("edit_form_endDate_4_day",endDate['days'])
        t.select("edit_form_endDate_4_hour",endDate['hours'])
        t.select("edit_form_endDate_4_minute",endDate['minutes'])
        t.select("eventType",eventtype)
        t.clickAndWait( "form_submit")
        return t

    def getSuite(self):
        """Get Suite"""
        return self.suite


class PloneSeleniumTool(PropertyManager, UniqueObject, Folder, ActionProviderBase):
    """
    PloneSeleniumTool
    """

    id = 'portal_selenium'
    meta_type = 'PloneSelenium Tool'
    _actions = []
    security = ClassSecurityInfo()
    
    _properties=({'id':'title', 'type': 'string', 'mode': 'w'},
                 {'id':'domains', 'type': 'lines', 'mode': 'w'},
                )

    manage_options=(
        (
        { 'label' : 'Overview'
        , 'action' : 'manage_overview'
        },
        ) + 
        ActionProviderBase.manage_options + 
        PropertyManager.manage_options + 
        Folder.manage_options
        )

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_overview')
    manage_overview = PageTemplateFile('manage_overview', _www)

    #
    #   ZMI methods
    #

    def __init__(self):
        # Properties to be edited by site manager
        # Properties are initialized in manage_afterAdd
        self.setDomains(("127.0.0.1",))

    def getPloneVersion(self):
        """Get the Plone Version"""
        return self.Control_Panel.Products.CMFPlone.version
        
    def getSuite(self):
        suite = Suite() 
        portal_url = getToolByName(self,'portal_url')
        portal = portal_url.getPortalObject()
        url=portal.absolute_url()
        base=urlparse(url)[2]
        suite.setBase(base)
        return suite

    security.declarePrivate('initProperties')
    def initProperties(self):
        """Initialize properties
        """
        pass

    def enumTestSuites(self):
        """ Enumeration of Selenium Functional Test Suites
        """
        utool = portal=getToolByName(self,'portal_url')
        portal = utool.getPortalObject()
        if not portal:
            raise "Can't get the portal object."
        context=createExprContext(self,portal,self)
        res = []
        index=0
        ids = []
        delete_indexes = []
        for a in self.listActions():
            action = a.getAction(context)
            if action['id'] in ids:
                delete_indexes.append(index)                                
            else:
                ids.append(action['id'])
                if a.testCondition(context):
                    try:
                        # We just want to know whether the script is there
                        scriptName = extractSuiteScriptName(action['url'])
                        ftest = self.restrictedTraverse(scriptName)
#                        ftest = self.restrictedTraverse('get_' + action['id'].lower() + '_ftests')
                        res.append(action)
                    except NotFound, Unauthorized:
                        # Broken
                        delete_indexes.append(index)                                
                index +=1                
        if delete_indexes:
            self.deleteActions(selections=delete_indexes)
                        
        res.sort(lambda a,b:cmp(a['name'],b['name']))
        return res

    def getFTest(self, name):
        for a in self.enumTestSuites():
            if a['name']==name:
                return a
        raise "No FTest by that name: %s" %name

    def addUser(self, id, fullname, roles):
        """ Adds a dummy user for the functional tests. """
        portal = getToolByName(self, 'portal_url').getPortalObject()
        # Using a technique to become omnipotent user taken from:
        # http://zopelabs.com/cookbook/1091472367
        # This is necessarry in order to be able to add users even 
        # when the tests are called by anonymous
        _policy=PermissiveSecurityPolicy()
        _oldpolicy=setSecurityPolicy(_policy)
        newSecurityManager(None, OmnipotentUser().__of__(portal.acl_users))
        pm = getToolByName(self, 'portal_membership')
        pr = getToolByName(self, 'portal_registration')
        sampleuser = {'username':id,
                      'fullname':fullname,
                      'email':'%s@example.com'%id,
                      'wysiwyg_editor':'', #Avoid problems with Kupu etc.
                      }
        if pm.getMemberById(id) == None:
            try:
                pm.addMember(id=id, password='admin', roles=roles, domains=self.domains, properties=sampleuser)
            except:
                pass
        else:
            try:
                pr.editMember(member_id=id, password='admin', roles=roles, domains=self.domains, properties=sampleuser)
            except:
                # XXX This is a hack to circumvent field.checkPermission("edit", self) in CMFMember
                # XXX We do the work of editMember manually here
                member = pm.getMemberById(id)
                member.update(**sampleuser)
                member.setSecurityProfile(password='admin',roles=roles,domains=self.domains)
        setSecurityPolicy(_oldpolicy)

    def addGroup(self, id, roles):
        """ Adds a dummy group for the functional tests. """
        portal = getToolByName(self, 'portal_url').getPortalObject()
        #portal.acl_users.Groups.acl_users.userFolderAddUser(id, password="admin", roles=roles, domains=self.domains)
        # Using a technique to become omnipotent user taken from:
        # http://zopelabs.com/cookbook/1091472367
        # This is necessarry in order to be able to add groups even 
        # when the tests are called by anonymous
        _policy=PermissiveSecurityPolicy()
        _oldpolicy=setSecurityPolicy(_policy)
        newSecurityManager(None, OmnipotentUser().__of__(portal.acl_users))
        pg = getToolByName(self, 'portal_groups')

        # Don't create the workspaces.
        flag=0
        try:
            if pg.groupWorkspacesCreationFlag:
                flag=pg.groupWorkspacesCreationFlag
                pg.groupWorkspacesCreationFlag=0
        except:
            pass

        if pg.getGroupById(id)==None:
            pg.addGroup(id, roles=roles, domains=self.domains)
        else:
            pg.editGroup(id, password='admin', roles = roles, permissions=())

        if flag:
            pg.groupWorkspacesCreationFlag=flag
        setSecurityPolicy(_oldpolicy)

    def setDomains(self, domains = ("127.0.0.1",)):
        self.domains=domains

    def importTest(self, ftest_name, suite_name, test_name):
        scriptName = self.getSuiteScriptName(ftest_name)
        #result = self.restrictedTraverse('get_'+ftest_name.lower()+'_ftests')()      
        result = self.restrictedTraverse(scriptName)()      
        f = FTest(base='')
        suite=result.getSuite()
        for s in suite:
            if s['title']==suite_name:
                for t in s['tests']:
                    if t['title']==test_name:
                        f.ftest=t['steps']
        return f

    def getSuiteScriptName(self, ftest_name):
        utool = portal=getToolByName(self,'portal_url')
        portal = utool.getPortalObject()
        if not portal:
            raise "Can't get the portal object."
        context=createExprContext(self,portal,self)
        for a in self.listActions():
            action = a.getAction(context)
            if action['name'] == ftest_name:
               result = extractSuiteScriptName(action['url'])
               return result
        
    def importTestByIndex(self, ftest_name, suite_index, test_index):
        scriptName = self.getSuiteScriptName(ftest_name)
        #result = self.restrictedTraverse('get_'+ftest_name.lower()+'_ftests')()      
        result = self.restrictedTraverse(scriptName)()      
        suites=result.getSuite()
        suite=suites[suite_index]
        tests=suite['tests']
        test=tests[test_index]
        return test    
        
    def getTestSuiteRegistry(self):
        tsreg = TestSuiteRegistry()
        for f in self.enumTestSuites():
            result=None
            try:
                scriptName = extractSuiteScriptName(f['action'])
#                result = self.restrictedTraverse('get_'+f['name'].lower()+'_ftests')()      
                result = self.restrictedTraverse(scriptName)()      
            except:
                pass
            if result:
                suite=result.getSuite()
                tsreg.addSuite(suite=suite, name=f['name'])
        return tsreg

class TestSuiteRegistry:
    __allow_access_to_unprotected_subobjects__ = 1
    def __init__(self):
        self.suites = []       
    def addSuite(self, suite, name):
        suite = RegisteredTestSuite(suite, name)
        self.suites.append(suite)
    def getSuites(self):
        """getSuites"""
        return self.suites
    def getSuite(self, name):
        """getSuite(name)"""
        result=None
        for suite in self.getSuites():
            if suite.getName()==name:
                result=suite
        return result

class RegisteredTestSuite:
    __allow_access_to_unprotected_subobjects__ = 1
    def __init__(self, ftests, name):
        self.name = name
        self.ftests = []
        for ftest in ftests:
            t = RegisteredFTests(name=ftest['title'], ftests=ftest['tests'])
            self.ftests.append(t)
    def getName(self):
        return self.name
    def getFTests(self):
        """getFTests"""
        return self.ftests

class RegisteredFTests:
    __allow_access_to_unprotected_subobjects__ = 1
    def __init__(self, name, ftests):
        self.name = name
        self.ftests = []
        for ftest in ftests:
            t = RegisteredFTest(name=ftest['title'], commands=ftest['steps'])
            self.ftests.append(t)
    def getName(self):
        return self.name
    def getFTests(self):
        return self.ftests

class RegisteredFTest:
    __allow_access_to_unprotected_subobjects__ = 1
    def __init__(self, name, commands):
        self.name = name
        self.commands = []
        for command in commands:
            c = RegisteredCommand(command[0],command[1],command[2])
            self.commands.append(c)
    def getName(self):
        return self.name
    def getCommands(self):
        return self.commands

class RegisteredCommand:
    __allow_access_to_unprotected_subobjects__ = 1
    def __init__(self, command, target, value):
        self.command = command
        self.target = target
        self.value = value
    def getCommand(self):
        return self.command
    def getTarget(self):
        return self.target
    def getValue(self):
        return self.value
        
                        
InitializeClass(PloneSeleniumTool)
